@@ -58,7 +58,7 @@ static unsigned int adobe_sizes = 0x0403165D;
 /* Device driver actions */
 static void GTK_Activate(NewDevDesc *dd);
 static void GTK_Circle(double x, double y, double r,
-		       int col, int fill, double gamma, int lty, double lwd,
+		       R_GE_gcontext *gc,
 		       NewDevDesc *dd);
 static void GTK_Clip(double x0, double x1, double y0, double y1, 
 		     NewDevDesc *dd);
@@ -67,30 +67,32 @@ static void GTK_Deactivate(NewDevDesc *dd);
 static void GTK_Hold(NewDevDesc *dd);
 static Rboolean GTK_Locator(double *x, double *y, NewDevDesc *dd);
 static void GTK_Line(double x1, double y1, double x2, double y2,
-		     int col, double gamma, int lty, double lwd,
+		     R_GE_gcontext *gc,
 		     NewDevDesc *dd);
-static void GTK_MetricInfo(int c, int font, double cex, double ps,
-			      double* ascent, double* descent,
-			      double* width, NewDevDesc *dd);
+static void GTK_MetricInfo(int c,
+			   R_GE_gcontext *gc,
+			   double* ascent, double* descent,
+			   double* width, NewDevDesc *dd);
 static void GTK_Mode(int mode, NewDevDesc *dd);
-static void GTK_NewPage(int fill, double gamma, NewDevDesc *dd);
+static void GTK_NewPage(R_GE_gcontext *gc, NewDevDesc *dd);
 static void GTK_Polygon(int n, double *x, double *y, 
-			int col, int fill, double gamma, int lty, double lwd,
+			R_GE_gcontext *gc,
 			NewDevDesc *dd);
 static void GTK_Polyline(int n, double *x, double *y, 
-			    int col, double gamma, int lty, double lwd,
-			    NewDevDesc *dd);
+			 R_GE_gcontext *gc,
+			 NewDevDesc *dd);
 static void GTK_Rect(double x0, double y0, double x1, double y1,
-		     int col, int fill, double gamma, int lty, double lwd,
+		     R_GE_gcontext *gc,
 		     NewDevDesc *dd);
 static void GTK_Size(double *left, double *right,
 		     double *bottom, double *top,
 		     NewDevDesc *dd);
-static double GTK_StrWidth(char *str, int font,
-			      double cex, double ps, NewDevDesc *dd);
+static double GTK_StrWidth(char *str,
+			   R_GE_gcontext *gc,
+			   NewDevDesc *dd);
 static void GTK_Text(double x, double y, char *str, 
 		     double rot, double hadj, 
-		     int col, double gamma, int font, double cex, double ps,
+		     R_GE_gcontext *gc,
 		     NewDevDesc *dd);
 static Rboolean GTK_Open(NewDevDesc*, gtkDesc*, char*, double, double);
 
@@ -554,21 +556,23 @@ static Rboolean GTK_Open(NewDevDesc *dd, gtkDesc *gtkd, char *dsp, double w, dou
     return TRUE;
 }
 
-static double GTK_StrWidth(char *str, int font,
-			      double cex, double ps, NewDevDesc *dd)
+static double GTK_StrWidth(char *str,
+			   R_GE_gcontext *gc,
+			   NewDevDesc *dd)
 {
     int size;
     gtkDesc *gtkd = (gtkDesc *) dd->deviceSpecific;
 
-    size = cex * ps + 0.5;
-    SetFont(dd, font, size);
+    size = gc->cex * gc->ps + 0.5;
+    SetFont(dd, gc->fontface, size);
 
     return (double) gdk_string_width(gtkd->font, str);
 }
 
-static void GTK_MetricInfo(int c, int font, double cex, double ps,
-			      double* ascent, double* descent,
-			      double* width, NewDevDesc *dd)
+static void GTK_MetricInfo(int c,
+			   R_GE_gcontext *gc,
+			   double* ascent, double* descent,
+			   double* width, NewDevDesc *dd)
 {
     gint size;
     gint lbearing, rbearing, iascent, idescent, iwidth;
@@ -576,8 +580,8 @@ static void GTK_MetricInfo(int c, int font, double cex, double ps,
     gchar tmp[2];
     gtkDesc *gtkd = (gtkDesc *) dd->deviceSpecific;
 
-    size = cex * ps + 0.5;
-    SetFont(dd, font, size);
+    size = gc->cex * gc->ps + 0.5;
+    SetFont(dd, gc->fontface, size);
 
     if(c == 0) {
 	maxwidth = 0;
@@ -625,7 +629,7 @@ static void GTK_Clip(double x0, double x1, double y0, double y1, NewDevDesc *dd)
        but not to the backing pixmap. This is a GTK+ bug that is
        unlikely to be fixed in this version (9 Jul 2002) - MTP
     */
-    /* gdk_gc_set_clip_rectangle(gtkd->wgc, &gtkd->clip); */
+     gdk_gc_set_clip_rectangle(gtkd->wgc, &gtkd->clip); 
 }
 
 static void GTK_Size(double *left, double *right,
@@ -668,7 +672,8 @@ static void GTK_resize(NewDevDesc *dd)
 }
 
 /* clear the drawing area */
-static void GTK_NewPage(int fill, double gamma, NewDevDesc *dd)
+static void GTK_NewPage(R_GE_gcontext *gc,
+			NewDevDesc *dd)
 {
     gtkDesc *gtkd;
 
@@ -681,11 +686,10 @@ static void GTK_NewPage(int fill, double gamma, NewDevDesc *dd)
 
     if(gtkd->drawing->window == NULL)
 	return;
-    if(gtkd->fill != fill && R_OPAQUE(fill)) {
-	SetColor(&gtkd->gcol_bg, fill);
-	gtkd->fill = fill;
-	gdk_window_set_background(gtkd->drawing->window, &gtkd->gcol_bg);
-    }
+
+    gtkd->fill = R_OPAQUE(gc->fill) ? gc->fill : R_RGB(255,255,255);
+    SetColor(&gtkd->gcol_bg, gtkd->fill);
+    gdk_window_set_background(gtkd->drawing->window, &gtkd->gcol_bg);
 
     gdk_window_clear(gtkd->drawing->window);
 
@@ -757,7 +761,7 @@ static void GTK_Deactivate(NewDevDesc *dd)
 /* drawing stuff */
 
 static void GTK_Rect(double x0, double y0, double x1, double y1,
-		     int col, int fill, double gamma, int lty, double lwd,
+		     R_GE_gcontext *gc,
 		     NewDevDesc *dd)
 {
     double tmp;
@@ -780,11 +784,11 @@ static void GTK_Rect(double x0, double y0, double x1, double y1,
     }
 
 
-    if (R_OPAQUE(fill)) {
-	SetColor(&gcol_fill, fill);
+    if (R_OPAQUE(gc->fill)) {
+	SetColor(&gcol_fill, gc->fill);
 	gdk_gc_set_foreground(gtkd->wgc, &gcol_fill);
 
-	SetLineType(dd, lty, lwd);
+	SetLineType(dd, gc->lty, gc->lwd);
 
 	gdk_draw_rectangle(gtkd->drawing->window,
 			   gtkd->wgc, TRUE,
@@ -797,11 +801,11 @@ static void GTK_Rect(double x0, double y0, double x1, double y1,
 			   (gint) x1 - (gint) x0,
 			   (gint) y1 - (gint) y0);
     }
-    if (R_OPAQUE(col)) {
-	SetColor(&gcol_outline, col);
+    if (R_OPAQUE(gc->col)) {
+	SetColor(&gcol_outline, gc->col);
 	gdk_gc_set_foreground(gtkd->wgc, &gcol_outline);
 
-	SetLineType(dd, lty, lwd);
+	SetLineType(dd, gc->lty, gc->lwd);
 
 	gdk_draw_rectangle(gtkd->drawing->window,
 			   gtkd->wgc, FALSE,
@@ -817,7 +821,7 @@ static void GTK_Rect(double x0, double y0, double x1, double y1,
 }
 
 static void GTK_Circle(double x, double y, double r,
-		       int col, int fill, double gamma, int lty, double lwd,
+		       R_GE_gcontext *gc,
 		       NewDevDesc *dd)
 {
     GdkColor gcol_fill, gcol_outline;
@@ -831,8 +835,8 @@ static void GTK_Circle(double x, double y, double r,
     iy = y - r;
     ir = 2 * floor(r + 0.5);
 
-    if (R_OPAQUE(fill)) {
-	SetColor(&gcol_fill, fill);
+    if (R_OPAQUE(gc->fill)) {
+	SetColor(&gcol_fill, gc->fill);
 	gdk_gc_set_foreground(gtkd->wgc, &gcol_fill);
 
 	gdk_draw_arc(gtkd->drawing->window,
@@ -844,11 +848,11 @@ static void GTK_Circle(double x, double y, double r,
 		     ix, iy, ir, ir,
 		     0, 23040);
     }
-    if (R_OPAQUE(col)) {
-	SetColor(&gcol_outline, col);
+    if (R_OPAQUE(gc->col)) {
+	SetColor(&gcol_outline, gc->col);
 	gdk_gc_set_foreground(gtkd->wgc, &gcol_outline);
 
-	SetLineType(dd, lty, lwd);
+	SetLineType(dd, gc->lty, gc->lwd);
 
 	gdk_draw_arc(gtkd->drawing->window,
 		     gtkd->wgc, FALSE,
@@ -862,7 +866,7 @@ static void GTK_Circle(double x, double y, double r,
 }
 
 static void GTK_Line(double x1, double y1, double x2, double y2,
-		     int col, double gamma, int lty, double lwd,
+		     R_GE_gcontext *gc,
 		     NewDevDesc *dd)
 {
     gtkDesc *gtkd = (gtkDesc *) dd->deviceSpecific;
@@ -875,11 +879,11 @@ static void GTK_Line(double x1, double y1, double x2, double y2,
     ix1 = (gint) x1;  iy1 = (gint) y1;
     ix2 = (gint) x2;  iy2 = (gint) y2;
 
-    if (R_OPAQUE(col)) {
-	SetColor(&gcol_fill, col);
+    if (R_OPAQUE(gc->col)) {
+	SetColor(&gcol_fill, gc->col);
 	gdk_gc_set_foreground(gtkd->wgc, &gcol_fill);
 
-	SetLineType(dd, lty, lwd);
+	SetLineType(dd, gc->lty, gc->lwd);
 
 	gdk_draw_line(gtkd->drawing->window,
 		      gtkd->wgc, ix1, iy1, ix2, iy2);
@@ -889,8 +893,8 @@ static void GTK_Line(double x1, double y1, double x2, double y2,
 }
 
 static void GTK_Polyline(int n, double *x, double *y, 
-			    int col, double gamma, int lty, double lwd,
-			    NewDevDesc *dd)
+			 R_GE_gcontext *gc,
+			 NewDevDesc *dd)
 {
     gtkDesc *gtkd = (gtkDesc *) dd->deviceSpecific;
     GdkColor gcol_fill;
@@ -907,11 +911,11 @@ static void GTK_Polyline(int n, double *x, double *y,
 	points[i].y = (gint16) y[i];
     }
 
-    if (R_OPAQUE(col)) {
-	SetColor(&gcol_fill, col);
+    if (R_OPAQUE(gc->col)) {
+	SetColor(&gcol_fill, gc->col);
 	gdk_gc_set_foreground(gtkd->wgc, &gcol_fill);
 
-	SetLineType(dd, lty, lwd);
+	SetLineType(dd, gc->lty, gc->lwd);
 
 	gdk_draw_lines(gtkd->drawing->window,
 		       gtkd->wgc, points, n);
@@ -923,7 +927,7 @@ static void GTK_Polyline(int n, double *x, double *y,
 }
 
 static void GTK_Polygon(int n, double *x, double *y, 
-			int col, int fill, double gamma, int lty, double lwd,
+			R_GE_gcontext *gc,
 			NewDevDesc *dd)
 {
     gtkDesc *gtkd = (gtkDesc *) dd->deviceSpecific;
@@ -941,8 +945,8 @@ static void GTK_Polygon(int n, double *x, double *y,
 	points[i].y = (gint16) y[i];
     }
 
-    if (R_OPAQUE(fill)) {
-	SetColor(&gcol_fill, fill);
+    if (R_OPAQUE(gc->fill)) {
+	SetColor(&gcol_fill, gc->fill);
 	gdk_gc_set_foreground(gtkd->wgc, &gcol_fill);
 
 	gdk_draw_polygon(gtkd->drawing->window,
@@ -950,11 +954,11 @@ static void GTK_Polygon(int n, double *x, double *y,
 	gdk_draw_polygon(gtkd->pixmap,
 			 gtkd->wgc, TRUE, points, n);
     }
-    if (R_OPAQUE(col)) {
-	SetColor(&gcol_outline, col);
+    if (R_OPAQUE(gc->col)) {
+	SetColor(&gcol_outline, gc->col);
 	gdk_gc_set_foreground(gtkd->wgc, &gcol_outline);
 
-	SetLineType(dd, lty, lwd);
+	SetLineType(dd, gc->lty, gc->lwd);
 
 	gdk_draw_polygon(gtkd->drawing->window,
 			 gtkd->wgc, FALSE, points, n);
@@ -967,7 +971,7 @@ static void GTK_Polygon(int n, double *x, double *y,
 
 static void GTK_Text(double x, double y, char *str, 
 		     double rot, double hadj, 
-		     int col, double gamma, int font, double cex, double ps,
+		     R_GE_gcontext *gc,
 		     NewDevDesc *dd)
 {
     gtkDesc *gtkd = (gtkDesc *) dd->deviceSpecific;
@@ -977,12 +981,12 @@ static void GTK_Text(double x, double y, char *str,
 
     if(!gtkd->drawing->window)
 	return;
-    size = cex * ps + 0.5;
-    SetFont(dd, font, size);
+    size = gc->cex * gc->ps + 0.5;
+    SetFont(dd, gc->fontface, size);
     gdk_gc_set_font(gtkd->wgc, gtkd->font);
 
-    if (R_OPAQUE(col)) {
-	SetColor(&gcol_fill, col);
+    if (R_OPAQUE(gc->col)) {
+	SetColor(&gcol_fill, gc->col);
 	gdk_gc_set_foreground(gtkd->wgc, &gcol_fill);
 
 	gdk_draw_text_rot(gtkd->drawing->window,
